C   Sector Primitive Model
C
C     by Naoki Sato  August 18, 1998

C ########################################
C      SETTING THE PARAMETERS
C ########################################

      PARAMETER (LX=72,MX=36,NX=5)

      PARAMETER (IX=360*50,IO=360,DTM=0.240E3)

      REAL DX(MX),DY(MX),FC(MX),DZ(NX),
     +     U0(LX,MX,NX),V0(LX,MX,NX),T0(LX,MX,NX),P0(LX,MX),
     +     U (LX,MX,NX),V (LX,MX,NX),T (LX,MX,NX),P (LX,MX),
     +     UP(LX,MX,NX),VP(LX,MX,NX),TP(LX,MX,NX),PP(LX,MX),
     +     DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX),DP(LX,MX),
     +     H0(LX,MX,NX),H (LX,MX,NX),W (LX,MX,NX),DM(LX,MX,NX)

C ########################################
C   1. PREPARATION
C ########################################

      CALL PARAM (LX,MX,NX,DX,DY,DZ,FC)
      CALL BASIC (LX,MX,NX,DY,DZ,FC,U0,V0,T0,P0)
      CALL INITZ (LX,MX,NX,U,V,T,P,UP,VP,TP,PP)
      CALL DIAGH (LX,MX,NX,DZ,T0,H0)

C   Output files are opened.

C   u.dat: Zonal wind anomaly
C   v.dat: Meridional wind anomaly
C   t.dat: Potential temperature anomaly
C   z.dat: Geopotential height anomaly
C   p.dat: Surface pressure anomaly

      OPEN(11,FILE='u.dat',STATUS='UNKNOWN',
     +     FORM='UNFORMATTED',ACCESS='DIRECT',RECL=4*LX*MX*NX)
      OPEN(12,FILE='v.dat',STATUS='UNKNOWN',
     +     FORM='UNFORMATTED',ACCESS='DIRECT',RECL=4*LX*MX*NX)
      OPEN(13,FILE='t.dat',STATUS='UNKNOWN',
     +     FORM='UNFORMATTED',ACCESS='DIRECT',RECL=4*LX*MX*NX)
      OPEN(14,FILE='z.dat',STATUS='UNKNOWN',
     +     FORM='UNFORMATTED',ACCESS='DIRECT',RECL=4*LX*MX*NX)
      OPEN(15,FILE='p.dat',STATUS='UNKNOWN',
     +     FORM='UNFORMATTED',ACCESS='DIRECT',RECL=4*LX*MX)

C #### Do loop 11 starts here ####

C   In roop 11, the time derivateves of U, V, T and P are calculated,
C   and time-integrations are performed for them.

      DO 11 I=1,IX

C ########################################
C   2. CALCULATION OF DEPENDENT VARIABLES
C ########################################

      CALL DIAGW (LX,MX,NX,DX,DY,DZ,U,V,W)
      CALL DIAGH (LX,MX,NX,DZ,T,H)

C ########################################
C   3. DYNAMICAL PROCESSES
C ########################################

      CALL TENDZ (LX,MX,NX,DU,DV,DT,DP)
      CALL CNVRG (LX,MX,NX,DX,DY,DZ,U,V,DP)
      CALL ADVEC (LX,MX,NX,DX,DY,DZ,'N',U0,V0,T0,U,V,W,T,DU,DV,DT,DM)
      CALL PGRAD (LX,MX,NX,DX,DY,'N',P0,H0,P,H,DU,DV)
      CALL CORIO (LX,MX,NX,'N',FC,U0,V0,U,V,DU,DV)
      CALL INTEL (LX,MX,NX,DTM,DU,DV,DT,DP,UP,VP,TP,PP,U,V,T,P)
      CALL POLES (LX,MX,NX,U,V,T,P)

C ########################################
C   4. PHYSICAL PROCESSES
C ########################################

      CALL TENDZ (LX,MX,NX,DU,DV,DT,DP)
      CALL DIFFU (LX,MX,NX,DX,DY,DZ,'N',U0,V0,T0,U,V,T,DU,DV,DT)
      CALL DMPNG (LX,MX,NX,U,V,T,DU,DV,DT)
      CALL INTEE (LX,MX,NX,DTM,DU,DV,DT,DP,UP,VP,TP,PP,U,V,T,P)
      CALL POLES (LX,MX,NX,U,V,T,P)

C ########################################
C   5. OUTPUT
C ########################################

      IF (MOD(I,IO).EQ.0) THEN
        WRITE(6,'(1X,I4,1X,A1,1X,I4)') I/IO,'/',IX/IO
        WRITE(11,REC=I/IO) U
        WRITE(12,REC=I/IO) V
        WRITE(13,REC=I/IO) T
        WRITE(14,REC=I/IO) H
        WRITE(15,REC=I/IO) P
      ENDIF

   11 CONTINUE

C #### Do loop 11 ends here ####

      CLOSE(11)
      CLOSE(12)
      CLOSE(13)
      CLOSE(14)
      CLOSE(15)

      STOP
      END

C ########################################
C   SUBROUTINES
C ########################################

C ========================================
C   INITIAL CONDITIONS
C ========================================

      SUBROUTINE PARAM (LX,MX,NX,DX,DY,DZ,FC)
      REAL DX(MX),DY(MX),DZ(NX),FC(MX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      DO 11 M=1,MX
        DX(M) = 0.5E0 * PI*RE/REAL(MX)
     +          * COS(0.5E0*PI*(REAL(M)-0.5E0)/REAL(MX))
        DY(M) = 0.5E0 * PI*RE/REAL(MX)
        FC(M) = 0.2E1 * OM * SIN(0.5E0*PI*(REAL(M)-0.5E0)/REAL(MX))
   11 CONTINUE
      DO 12 N=1,NX
        R = (1.0E0/0.15E0)**(1.0E0/REAL(NX))
        DZ(N) = 0.15E0*(R-1.0E0)*R**(N-1)
   12 CONTINUE
      RETURN
      END

      SUBROUTINE BASIC (LX,MX,NX,DY,DZ,FC,U0,V0,T0,P0)
      REAL DY(MX),DZ(NX),FC(MX),
     +     U0(LX,MX,NX),V0(LX,MX,NX),T0(LX,MX,NX),P0(LX,MX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      TM00 = T00*(P00/1.0E5)**(0.2E1/0.7E1)
      H00 = RC*TM00/GC
      H0 = - H00 * ALOG(0.25E0)
      Y0 = 0.5E0*RE
      TH0 = 278.
      TH1 = 20.
      DO 11 N=1,NX
      DO 12 M=1,MX
      THETA = PI*(REAL(M)-0.5E0)/REAL(MX)
      FC1 = AMAX1(FC(M),0.5E0*OM)
      DO 13 L=1,LX
      CALL SIGMA (NX,DZ,N,Z)
      H = - H00 * ALOG(Z)
        U0(L,M,N) = TH1 * 0.25E0*GC*H00/FC1/T00/Y0 * SIN(THETA)
     +     * (ALOG(1.+SINH(H0/(0.5E0*H00))**2)
     +        -ALOG(1.+SINH((H0-H)/(0.5E0*H00))**2))
        V0(L,M,N) = 0.0E0
        T0(L,M,N) = TH0 + 0.5E-2 * H
        IF (H.GT.H0) T0(L,M,N) = T0(L,M,N) + 0.5E-2 * (H-H0)
        T0(L,M,N) = T0(L,M,N)
     +               + TH1 * COS(THETA) * TANH((H0-H)/(0.5E0*H00))
        P0(L,M) = P00
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE INITZ (LX,MX,NX,U,V,T,P,UP,VP,TP,PP)
      REAL U (LX,MX,NX),V (LX,MX,NX),T (LX,MX,NX),P (LX,MX),
     +     UP(LX,MX,NX),VP(LX,MX,NX),TP(LX,MX,NX),PP(LX,MX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      DO 11 N=1,NX
      DO 12 M=1,MX
      DO 13 L=1,LX
        U (L,M,N) = 0.0E0
        V (L,M,N) = 0.0E0
        T (L,M,N) = 0.0E0
      IF ((L.LE.LX/4).AND.(M.GE.MX/4+1).AND.(M.LE.3*MX/4)
     +    .AND.(N.GE.3).AND.(N.LE.4))
     +  T (L,M,N) = T(L,M,N)
     +    - 5. * SIN(4.*PI*REAL(L-1)/REAL(LX))
     +         * COS(2.*PI*(REAL(M)-0.5)/REAL(MX))
        P (L,M) = 0.0E0
        UP(L,M,N) = U(L,M,N)
        VP(L,M,N) = V(L,M,N)
        TP(L,M,N) = T(L,M,N)
        PP(L,M) = P(L,M)
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

C ========================================

      SUBROUTINE POLES (LX,MX,NX,U,V,T,P)
      REAL U(LX,MX,NX),V(LX,MX,NX),T(LX,MX,NX),P(LX,MX)
      CALL POLESG (LX,MX,NX,T)
      CALL POLESG (LX,MX,NX,U)
      CALL POLESV (LX,MX,NX,V)
      CALL POLESG (LX,MX, 1,P)
      RETURN
      END

      SUBROUTINE POLESG (LX,MX,NX,G)
      REAL G(LX,MX,NX)
      DO 11 M=25,30
        CALL POLESA (LX,MX,NX,M,2,G)
   11 CONTINUE
      DO 12 M=31,33
        CALL POLESA (LX,MX,NX,M,4,G)
   12 CONTINUE
      DO 13 M=34,34
        CALL POLESA (LX,MX,NX,M,8,G)
   13 CONTINUE
      DO 14 M=35,35
        CALL POLESA (LX,MX,NX,M,16,G)
   14 CONTINUE
      DO 15 M=36,36
        CALL POLESA (LX,MX,NX,M,LX,G)
   15 CONTINUE
      RETURN
      END

      SUBROUTINE POLESV (LX,MX,NX,G)
      REAL G(LX,MX,NX)
      DO 11 M=25,30
        CALL POLESA (LX,MX,NX,M,2,G)
   11 CONTINUE
      DO 12 M=31,33
        CALL POLESA (LX,MX,NX,M,4,G)
   12 CONTINUE
      DO 13 M=34,34
        CALL POLESA (LX,MX,NX,M,8,G)
   13 CONTINUE
      DO 14 M=35,35
        CALL POLESA (LX,MX,NX,M,16,G)
   14 CONTINUE
      DO 15 M=36,36
        CALL POLESA (LX,MX,NX,M,LX,G)
   15 CONTINUE
      RETURN
      END

      SUBROUTINE POLESA (LX,MX,NX,M,LR,G)
      REAL G(LX,MX,NX)
      DO 11 N=1,NX
      DO 12 K=1,(LX-1)/LR+1
      L1 = 1+LR*(K-1)
      L2 = MIN0(LR+LR*(K-1),LX)
        SUM = 0.0E0
      DO 21 L=L1,L2
        SUM = SUM + G(L,M,N)
   21 CONTINUE
      DO 22 L=L1,L2
        G(L,M,N) = SUM / REAL(L2-L1+1)
   22 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

C ========================================

      SUBROUTINE CONST (PI,RE,OM,GC,RC,T00,P00)
      PI = 0.31415926535E1
      RE = 0.637E7
      OM = 0.727E-4
      GC = 0.9807E1
      RC = 0.2870E3
      T00 = 0.298E3
      P00 = 0.101325E6
      RETURN
      END

      SUBROUTINE SIGMA (NX,DZ,N,Z)
      REAL DZ(NX)
        Z = 1.0E0
      DO 11 NN=NX,N+1,-1
        Z = Z - DZ(NN)
   11 CONTINUE
        Z = Z - 0.5E0*DZ(N)
      RETURN
      END

C ========================================
C   DIAGNOSES OF DEPENDENT VARIABLES
C ========================================

      SUBROUTINE DIAGW (LX,MX,NX,DX,DY,DZ,U,V,W)
      REAL DX(MX),DY(MX),DZ(NX),U(LX,MX,NX),V(LX,MX,NX),W(LX,MX,NX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      DO 11 M=1,MX
      MB = MOD(M+1+MX-1,MX)+1
      M1 = MAX0(M-1, 1)
      M2 = MIN0(M+1,MX)
      DO 12 L=1,LX
      LB = MOD(L+1+LX-1,LX)+1
        W0 = 0.0E0
      DO 21 N=1,NX
      DX1 = 0.5E0*(DX(M)+DX(M1))
      DX2 = 0.5E0*(DX(M)+DX(M2))
      DXDY = DX(M) * DY(M)
      FX1 = DY(M) * U(L ,M ,N)
      FX2 = DY(M) * U(LB,M ,N)
      FY1 = DX1   * V(L ,M ,N)
      FY2 = DX2   * V(L ,MB,N)
        W0 = W0 + (FX2-FX1+FY2-FY1)/DXDY * DZ(N)
   21 CONTINUE
      DO 31 N=1,NX-1
      CALL SIGMA (NX,DZ,N,Z)
        W1 = 0.0E0
      DO 32 NN=1,N
      FX1 = DY(M) * U(L ,M ,NN)
      FX2 = DY(M) * U(LB,M ,NN)
      FY1 = DX1   * V(L ,M ,NN)
      FY2 = DX2   * V(L ,MB,NN)
        W1 = W1 - (FX2-FX1+FY2-FY1)/DXDY * DZ(NN)
   32 CONTINUE
        W(L,M,N+1)= W1+Z*W0
   31 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE DIAGH (LX,MX,NX,DZ,T,H)
      REAL DZ(NX),T(LX,MX,NX),H(LX,MX,NX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      DO 11 M=1,MX
      DO 12 L=1,LX
        HH = 0.0E0
      DO 21 N=NX,1,-1
      CALL SIGMA (NX,DZ,N,Z)
        TM = T(L,M,N) * (P00/1.0E5*Z)**(0.2E1/0.7E1)
        DH = RC * TM / GC * DZ(N)
        H(L,M,N) = HH + 0.5*DH
        HH = HH + DH
   21 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

C ========================================
C   DYNAMICAL PROCESSES
C ========================================

      SUBROUTINE CNVRG (LX,MX,NX,DX,DY,DZ,U,V,DP)
      REAL DX(MX),DY(MX),DZ(NX),
     +     U(LX,MX,NX),V(LX,MX,NX),DP(LX,MX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      DO 11 M=1,MX
      MA = MOD(M-1+MX-1,MX)+1
      MB = MOD(M+1+MX-1,MX)+1
      DO 12 L=1,LX
      LB = MOD(L+1+LX-1,LX)+1
      DO 21 N=1,NX
      DX1 = 0.5E0*(DX(M)+DX(MA))
      DX2 = 0.5E0*(DX(M)+DX(MB))
      DXDY = DX(M) * DY(M)
      FX1 = DY(M) * U(L ,M ,N)
      FX2 = DY(M) * U(LB,M ,N)
      FY1 = DX1   * V(L ,M ,N)
      FY2 = DX2   * V(L ,MB,N)
        DP(L,M) = DP(L,M) - P00 * (FX2-FX1+FY2-FY1)/DXDY * DZ(N)
   21 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

C ========================================

      SUBROUTINE ADVEC (LX,MX,NX,DX,DY,DZ,CM,
     +                  U0,V0,T0,U,V,W,T,DU,DV,DT,ZR)
      REAL DX(MX),DY(MX),DZ(NX),
     +     U0(LX,MX,NX),V0(LX,MX,NX),T0(LX,MX,NX),
     +     U(LX,MX,NX),V(LX,MX,NX),W(LX,MX,NX),T(LX,MX,NX),
     +     DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX),ZR(LX,MX,NX)
      CHARACTER CM*1
      CALL ZERO   (LX,MX,NX,ZR)
      CALL ADVECU (LX,MX,NX,DX,DY,DZ,U0,V0,ZR,U ,DU)
      CALL ADVECV (LX,MX,NX,DX,DY,DZ,U0,V0,ZR,V ,DV) 
      CALL ADVECG (LX,MX,NX,DX,DY,DZ,U0,V0,ZR,T ,DT)
      CALL ADVECU (LX,MX,NX,DX,DY,DZ,U ,V ,W ,U0,DU)
      CALL ADVECV (LX,MX,NX,DX,DY,DZ,U ,V ,W ,V0,DV) 
      CALL ADVECG (LX,MX,NX,DX,DY,DZ,U ,V ,W ,T0,DT)
      IF (CM.EQ.'N') THEN
      CALL ADVECU (LX,MX,NX,DX,DY,DZ,U0,V0,ZR,U0,DU)
      CALL ADVECV (LX,MX,NX,DX,DY,DZ,U0,V0,ZR,V0,DV) 
      CALL ADVECG (LX,MX,NX,DX,DY,DZ,U0,V0,ZR,T0,DT)
      CALL ADVECU (LX,MX,NX,DX,DY,DZ,U ,V ,W ,U ,DU)
      CALL ADVECV (LX,MX,NX,DX,DY,DZ,U ,V ,W ,V ,DV) 
      CALL ADVECG (LX,MX,NX,DX,DY,DZ,U ,V ,W ,T ,DT)
      ENDIF
      RETURN
      END

      SUBROUTINE ADVECG (LX,MX,NX,DX,DY,DZ,U,V,W,G,DG)
      REAL DX(MX),DY(MX),DZ(NX),
     +     U(LX,MX,NX),V(LX,MX,NX),W(LX,MX,NX),
     +     G(LX,MX,NX),DG(LX,MX,NX)
      DO 11 N=1,NX
      NA = MOD(N-1+NX-1,NX)+1
      NB = MOD(N+1+NX-1,NX)+1
      DO 12 M=1,MX
      MA = MOD(M-1+MX-1,MX)+1
      MB = MOD(M+1+MX-1,MX)+1
      DX1 = 0.5E0*(DX(M)+DX(MA))
      DX2 = 0.5E0*(DX(M)+DX(MB))
      SX1 = DY(M) * DZ(N)
      SX2 = SX1
      SY1 = DX1 * DZ(N)
      SY2 = DX2 * DZ(N)
      SZ1 = DX(M) * DY(M)
      SZ2 = SZ1
      DXDYDZ = SZ1 * DZ(N)
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      LB = MOD(L+1+LX-1,LX)+1
      FX1 = SX1 * U(L ,M ,N ) * (G(L ,M ,N )-G(LA,M ,N ))
      FX2 = SX2 * U(LB,M ,N ) * (G(LB,M ,N )-G(L ,M ,N ))
      FY1 = SY1 * V(L ,M ,N ) * (G(L ,M ,N )-G(L ,MA,N ))
      FY2 = SY2 * V(L ,MB,N ) * (G(L ,MB,N )-G(L ,M ,N ))
      FZ1 = SZ1 * W(L ,M ,N ) * (G(L ,M ,N )-G(L ,M ,NA))
      FZ2 = SZ2 * W(L ,M ,NB) * (G(L ,M ,NB)-G(L ,M ,N ))
        DG(L,M,N) = DG(L,M,N)
     +    - 0.5E0 * (FX2+FX1+FY2+FY1+FZ2+FZ1) / DXDYDZ
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE ADVECU (LX,MX,NX,DX,DY,DZ,U,V,W,G,DG)
      REAL DX(MX),DY(MX),DZ(NX),
     +     U(LX,MX,NX),V(LX,MX,NX),W(LX,MX,NX),
     +     G(LX,MX,NX),DG(LX,MX,NX)
      DO 11 N=1,NX
      NA = MOD(N-1+NX-1,NX)+1
      NB = MOD(N+1+NX-1,NX)+1
      DO 12 M=1,MX
      MA = MOD(M-1+MX-1,MX)+1
      MB = MOD(M+1+MX-1,MX)+1
      DX1 = 0.5E0*(DX(M)+DX(MA))
      DX2 = 0.5E0*(DX(M)+DX(MB))
      SX1 = DY(M) * DZ(N)
      SX2 = SX1
      SY1 = DX1 * DZ(N)
      SY2 = DX2 * DZ(N)
      SZ1 = DX(M) * DY(M)
      SZ2 = SZ1
      DXDYDZ = SZ1 * DZ(N)
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      LB = MOD(L+1+LX-1,LX)+1
      U1 = 0.5E0*(U(LA,M ,N )+U(L ,M ,N ))
      U2 = 0.5E0*(U(L ,M ,N )+U(LB,M ,N ))
      V1 = 0.5E0*(V(LA,M ,N )+V(L ,M ,N ))
      V2 = 0.5E0*(V(LA,MB,N )+V(L ,MB,N ))
      W1 = 0.5E0*(W(LA,M ,N )+W(L ,M ,N ))
      W2 = 0.5E0*(W(LA,M ,NB)+W(L ,M ,NB))
      FX1 = SX1 * U1 * (G(L ,M ,N )-G(LA,M ,N ))
      FX2 = SX2 * U2 * (G(LB,M ,N )-G(L ,M ,N ))
      FY1 = SY1 * V1 * (G(L ,M ,N )-G(L ,MA,N ))
      FY2 = SY2 * V2 * (G(L ,MB,N )-G(L ,M ,N ))
      FZ1 = SZ1 * W1 * (G(L ,M ,N )-G(L ,M ,NA))
      FZ2 = SZ2 * W2 * (G(L ,M ,NB)-G(L ,M ,N ))
        DG(L,M,N) = DG(L,M,N)
     +    - 0.5E0 * (FX2+FX1+FY2+FY1+FZ2+FZ1) / DXDYDZ
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE ADVECV (LX,MX,NX,DX,DY,DZ,U,V,W,G,DG)
      REAL DX(MX),DY(MX),DZ(NX),
     +     U(LX,MX,NX),V(LX,MX,NX),W(LX,MX,NX),
     +     G(LX,MX,NX),DG(LX,MX,NX)
      DO 11 N=1,NX
      NA = MOD(N-1+NX-1,NX)+1
      NB = MOD(N+1+NX-1,NX)+1
      DO 12 M=2,MX
      MA = MOD(M-1+MX-1,MX)+1
      MB = MOD(M+1+MX-1,MX)+1
      DX0 = 0.5E0*(DX(M)+DX(MA))
      DY0 = 0.5E0*(DY(M)+DY(MA))
      SX1 = DY0 * DZ(N)
      SX2 = SX1
      SY1 = DX(MA) * DZ(N)
      SY2 = DX(M)  * DZ(N)
      SZ1 = DX0 * DY0
      SZ2 = SZ1
      DXDYDZ = SZ1 * DZ(N)
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      LB = MOD(L+1+LX-1,LX)+1
      U1 = 0.5E0*(U(L ,MA,N )+U(L ,M ,N ))
      U2 = 0.5E0*(U(LB,MA,N )+U(LB,M ,N ))
      V1 = 0.5E0*(V(L ,MA,N )+V(L ,M ,N ))
      V2 = 0.5E0*(V(L ,M ,N )+V(L ,MB,N ))
      W1 = 0.5E0*(W(L ,MA,N )+W(L ,M ,N ))
      W2 = 0.5E0*(W(L ,MA,NB)+W(L ,M ,NB))
      FX1 = SX1 * U1 * (G(L ,M ,N )-G(LA,M ,N ))
      FX2 = SX2 * U2 * (G(LB,M ,N )-G(L ,M ,N ))
      FY1 = SY1 * V1 * (G(L ,M ,N )-G(L ,MA,N ))
      FY2 = SY2 * V2 * (G(L ,MB,N )-G(L ,M ,N ))
      FZ1 = SZ1 * W1 * (G(L ,M ,N )-G(L ,M ,NA))
      FZ2 = SZ2 * W2 * (G(L ,M ,NB)-G(L ,M ,N ))
        DG(L,M,N) = DG(L,M,N)
     +    - 0.5E0 * (FX2+FX1+FY2+FY1+FZ2+FZ1) / DXDYDZ
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

C ========================================

      SUBROUTINE PGRAD (LX,MX,NX,DX,DY,CM,P0,H0,P,H,DU,DV)
      REAL DX(MX),DY(MX),
     +     P0(LX,MX),P(LX,MX),H0(LX,MX,NX),H(LX,MX,NX),
     +     DU(LX,MX,NX),DV(LX,MX,NX)
      CHARACTER CM*1
      CALL PGRAD1 (LX,MX,NX,DX,DY,P ,H ,DU,DV)
      IF (CM.EQ.'N') THEN
      CALL PGRAD1 (LX,MX,NX,DX,DY,P0,H0,DU,DV)
      ENDIF
      RETURN
      END

      SUBROUTINE PGRAD1 (LX,MX,NX,DX,DY,P,H,DU,DV)
      REAL DX(MX),DY(MX),
     +     P(LX,MX),H(LX,MX,NX),DU(LX,MX,NX),DV(LX,MX,NX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      TM00 = T00*(P00/1.0E5)**(0.2E1/0.7E1)
      R00 = P00/RC/TM00
      DO 11 N=1,NX
      DO 12 M=1,MX
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
        DU(L,M,N) = DU(L,M,N)
     +    - GC * (H(L,M,N)-H(LA,M,N)) / DX(M)
     +    - (P(L,M)-P(LA,M)) / DX(M) / R00
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      DO 21 L=1,LX
      DO 22 M=2,MX
      MA = MOD(M-1+MX-1,MX)+1
      DY0 = 0.5E0*(DY(MA)+DY(M))
      DO 23 N=1,NX
        DV(L,M,N) = DV(L,M,N)
     +    - GC * (H(L,M,N)-H(L,MA,N)) / DY0
     +    - (P(L,M)-P(L,MA)) / DY0 / R00
   23 CONTINUE
   22 CONTINUE
   21 CONTINUE
      RETURN
      END

C ========================================

      SUBROUTINE CORIO (LX,MX,NX,CM,FC,U0,V0,U,V,DU,DV)
      REAL FC(MX),U0(LX,MX,NX),V0(LX,MX,NX),U(LX,MX,NX),V(LX,MX,NX),
     +     DU(LX,MX,NX),DV(LX,MX,NX)
      CHARACTER CM*1
      CALL CORIO1 (LX,MX,NX,FC,U ,V ,DU,DV)
      IF (CM.EQ.'N') THEN
      CALL CORIO1 (LX,MX,NX,FC,U0,V0,DU,DV)
      ENDIF
      RETURN
      END

      SUBROUTINE CORIO1 (LX,MX,NX,FC,U,V,DU,DV) 
      REAL FC(MX),U(LX,MX,NX),V(LX,MX,NX),DU(LX,MX,NX),DV(LX,MX,NX)
      DO 11 N=1,NX
      DO 12 M=1,MX
      MB = MOD(M+1+MX-1,MX)+1
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      V0 = 0.25E0*(V(LA,M,N)+V(L,M,N)+V(LA,MB,N)+V(L,MB,N))
        DU(L,M,N) = DU(L,M,N) + FC(M) * V0
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      DO 21 N=1,NX
      DO 22 M=2,MX
      MA = MOD(M-1+MX-1,MX)+1
      F0 = 0.5E0*(FC(MA)+FC(M))
      DO 23 L=1,LX
      LB = MOD(L+1+LX-1,LX)+1
      U0 = 0.25E0*(U(L,MA,N)+U(LB,MA,N)+U(L,M,N)+U(LB,M,N))
        DV(L,M,N) = DV(L,M,N) - F0 * U0
   23 CONTINUE
   22 CONTINUE
   21 CONTINUE
      RETURN
      END

C ========================================
C   PHYSICAL PROCESSES
C ========================================

      SUBROUTINE DIFFU (LX,MX,NX,DX,DY,DZ,CM,U0,V0,T0,U,V,T,DU,DV,DT)
      REAL DX(MX),DY(MX),DZ(NX),
     +     U0(LX,MX,NX),V0(LX,MX,NX),T0(LX,MX,NX),
     +     U (LX,MX,NX),V (LX,MX,NX),T (LX,MX,NX),
     +     DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX)
      CHARACTER CM*1
      CALL DIFFU1 (LX,MX,NX,DX,DY,DZ,U ,V ,T ,DU,DV,DT)
      IF (CM.EQ.'N') THEN
      CALL DIFFU1 (LX,MX,NX,DX,DY,DZ,U0,V0,T0,DU,DV,DT)
      ENDIF
      RETURN
      END

      SUBROUTINE DIFFU1 (LX,MX,NX,DX,DY,DZ,U,V,T,DU,DV,DT)
      REAL DX(MX),DY(MX),DZ(NX),
     +     U (LX,MX,NX),V (LX,MX,NX),T (LX,MX,NX),
     +     DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX)
      DFH = 1.0E5
      DFV = 1.0E0
      CALL DIFFUU (LX,MX,NX,DX,DY,DZ,DFH,DFV,U,DU)
      CALL DIFFUV (LX,MX,NX,DX,DY,DZ,DFH,DFV,V,DV)
      CALL DIFFUG (LX,MX,NX,DX,DY,DZ,DFH,DFV,T,DT)
      RETURN
      END

      SUBROUTINE DIFFUG (LX,MX,NX,DX,DY,DZ,DFH,DFV,G,DG)
      REAL DX(MX),DY(MX),DZ(NX),G(LX,MX,NX),DG(LX,MX,NX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      TM00 = T00*(P00/1.0E5)**(0.2E1/0.7E1)
      H00 = RC*TM00/GC
      DO 11 N=1,NX
      DO 12 M=1,MX
      MA = MOD(M-1+MX-1,MX)+1
      MB = MOD(M+1+MX-1,MX)+1
      M1 = MAX(M-1, 1)
      M2 = MIN(M+1,MX)
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      LB = MOD(L+1+LX-1,LX)+1
        DG(L,M,N) = DG(L,M,N)
     +    + DFH * (G(LA,M,N)+G(LB,M,N)-0.2E1*G(L,M,N))
     +      / DX(M)**2
     +    + DFH * (G(L,M1,N)+G(L,M2,N)-0.2E1*G(L,M,N))
     +      / DY(M)**2
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      DO 21 N=1,NX
      NA = MOD(N-1+NX-1,NX)+1
      NB = MOD(N+1+NX-1,NX)+1
      N1 = MAX(N-1, 1)
      N2 = MIN(N+1,NX)
      CALL SIGMA (NX,DZ,N ,Z )
      CALL SIGMA (NX,DZ,N1,Z1)
      CALL SIGMA (NX,DZ,N2,Z2)
      DZ1 = 0.5E0*(DZ(N1)+DZ(N ))
      DZ2 = 0.5E0*(DZ(N )+DZ(N2))
      DFV0 = DFV / H00**2
      DFV1 = DFV0 * (0.5E0*(Z+Z1))**2
      DFV2 = DFV0 * (0.5E0*(Z+Z2))**2
      DO 22 M=1,MX
      DO 23 L=1,LX
        DG(L,M,N) = DG(L,M,N)
     +    + (  DFV2*(G(L,M,N2)-G(L,M,N))/DZ2
     +       - DFV1*(G(L,M,N1)-G(L,M,N))/DZ1)
     +      / DZ(N)
   23 CONTINUE
   22 CONTINUE
   21 CONTINUE
      RETURN
      END

      SUBROUTINE DIFFUU (LX,MX,NX,DX,DY,DZ,DFH,DFV,G,DG)
      REAL DX(MX),DY(MX),DZ(NX),G(LX,MX,NX),DG(LX,MX,NX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      TM00 = T00*(P00/1.0E5)**(0.2E1/0.7E1)
      H00 = RC*TM00/GC
      DO 11 N=1,NX
      DO 12 M=1,MX
      MA = MOD(M-1+MX-1,MX)+1
      MB = MOD(M+1+MX-1,MX)+1
      M1 = MAX(M-1, 1)
      M2 = MIN(M+1,MX)
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      LB = MOD(L+1+LX-1,LX)+1
        DG(L,M,N) = DG(L,M,N)
     +    + DFH * (G(LA,M,N)+G(LB,M,N)-0.2E1*G(L,M,N))
     +      / DX(M)**2
     +    + DFH * (G(L,M1,N)+G(L,M2,N)-0.2E1*G(L,M,N))
     +      / DY(M)**2
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      DO 21 N=1,NX
      NA = MOD(N-1+NX-1,NX)+1
      NB = MOD(N+1+NX-1,NX)+1
      N1 = MAX(N-1, 1)
      N2 = MIN(N+1,NX)
      CALL SIGMA (NX,DZ,N ,Z )
      CALL SIGMA (NX,DZ,N1,Z1)
      CALL SIGMA (NX,DZ,N2,Z2)
      DZ1 = 0.5E0*(DZ(N1)+DZ(N ))
      DZ2 = 0.5E0*(DZ(N )+DZ(N2))
      DFV0 = DFV / H00**2
      DFV1 = DFV0 * (0.5E0*(Z+Z1))**2
      DFV2 = DFV0 * (0.5E0*(Z+Z2))**2
      DO 22 M=1,MX
      DO 23 L=1,LX
        DG(L,M,N) = DG(L,M,N)
     +    + (  DFV2*(G(L,M,N2)-G(L,M,N))/DZ2
     +       - DFV1*(G(L,M,N1)-G(L,M,N))/DZ1)
     +      / DZ(N)
      IF (N.EQ.NX) THEN
        DG(L,M,N) = DG(L,M,N)
     +     - DFV2*G(L,M,N)/(0.5*DZ2) / DZ(N)
      ENDIF
   23 CONTINUE
   22 CONTINUE
   21 CONTINUE
      RETURN
      END

      SUBROUTINE DIFFUV (LX,MX,NX,DX,DY,DZ,DFH,DFV,G,DG)
      REAL DX(MX),DY(MX),DZ(NX),G(LX,MX,NX),DG(LX,MX,NX)
      CALL CONST (PI,RE,OM,GC,RC,T00,P00)
      TM00 = T00*(P00/1.0E5)**(0.2E1/0.7E1)
      H00 = RC*TM00/GC
      DO 11 N=1,NX
      DO 12 M=2,MX
      MA = MOD(M-1+MX-1,MX)+1
      MB = MOD(M+1+MX-1,MX)+1
      DX0 = 0.5E0*(DX(MA)+DX(M))
      DY0 = 0.5E0*(DY(MA)+DY(M))
      DO 13 L=1,LX
      LA = MOD(L-1+LX-1,LX)+1
      LB = MOD(L+1+LX-1,LX)+1
        DG(L,M,N) = DG(L,M,N)
     +    + DFH * (G(LA,M,N)+G(LB,M,N)-0.2E1*G(L,M,N))
     +      / DX0**2
     +    + DFH * (G(L,MA,N)+G(L,MB,N)-0.2E1*G(L,M,N))
     +      / DY0**2
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      DO 21 N=1,NX
      NA = MOD(N-1+NX-1,NX)+1
      NB = MOD(N+1+NX-1,NX)+1
      N1 = MAX(N-1, 1)
      N2 = MIN(N+1,NX)
      CALL SIGMA (NX,DZ,N ,Z )
      CALL SIGMA (NX,DZ,N1,Z1)
      CALL SIGMA (NX,DZ,N2,Z2)
      DZ1 = 0.5E0*(DZ(N1)+DZ(N ))
      DZ2 = 0.5E0*(DZ(N )+DZ(N2))
      DFV0 = DFV / H00**2
      DFV1 = DFV0 * (0.5E0*(Z+Z1))**2
      DFV2 = DFV0 * (0.5E0*(Z+Z2))**2
      DO 22 M=2,MX
      DO 23 L=1,LX
        DG(L,M,N) = DG(L,M,N)
     +    + (  DFV2*(G(L,M,N2)-G(L,M,N))/DZ2
     +       - DFV1*(G(L,M,N1)-G(L,M,N))/DZ1)
     +      / DZ(N)
      IF (N.EQ.NX) THEN
        DG(L,M,N) = DG(L,M,N)
     +     - DFV2*G(L,M,N)/(0.5*DZ2) / DZ(N)
      ENDIF
   23 CONTINUE
   22 CONTINUE
   21 CONTINUE
      RETURN
      END

C ========================================

      SUBROUTINE DMPNG (LX,MX,NX,U,V,T,DU,DV,DT)
      REAL U(LX,MX,NX),V(LX,MX,NX),T(LX,MX,NX),
     +     DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX)
      TAU = 1.0E6
      DO 11 N=1,NX
      DO 12 M=1,MX
      DO 13 L=1,LX
        DU(L,M,N) = DU(L,M,N) - 1.0E0 / TAU * U(L,M,N)
        DV(L,M,N) = DV(L,M,N) - 1.0E0 / TAU * V(L,M,N)
        DT(L,M,N) = DT(L,M,N) - 1.0E0 / TAU * T(L,M,N)
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

C ========================================
C   TEMPORAL EVOLUTION
C ========================================

      SUBROUTINE TENDZ (LX,MX,NX,DU,DV,DT,DP)
      REAL DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX),DP(LX,MX)
      CALL ZERO (LX,MX,NX,DU)
      CALL ZERO (LX,MX,NX,DV)
      CALL ZERO (LX,MX,NX,DT)
      CALL ZERO (LX,MX, 1,DP)
      RETURN
      END

      SUBROUTINE ZERO (LX,MX,NX,G)
      REAL G(LX,MX,NX)
      DO 11 N=1,NX
      DO 12 M=1,MX
      DO 13 L=1,LX
        G(L,M,N) = 0.0E0
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE INTEL (LX,MX,NX,DTM,DU,DV,DT,DP,UP,VP,TP,PP,U,V,T,P)
      REAL U (LX,MX,NX),V (LX,MX,NX),T (LX,MX,NX),P (LX,MX),
     +     UP(LX,MX,NX),VP(LX,MX,NX),TP(LX,MX,NX),PP(LX,MX),
     +     DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX),DP(LX,MX)
      CALL INTEL1 (LX,MX,NX,DTM,DU,UP,U)
      CALL INTEL1 (LX,MX,NX,DTM,DV,VP,V)
      CALL INTEL1 (LX,MX,NX,DTM,DT,TP,T)
      CALL INTEL1 (LX,MX, 1,DTM,DP,PP,P)
      RETURN
      END

      SUBROUTINE INTEE (LX,MX,NX,DTM,DU,DV,DT,DP,UP,VP,TP,PP,U,V,T,P)
      REAL U (LX,MX,NX),V (LX,MX,NX),T (LX,MX,NX),P (LX,MX),
     +     UP(LX,MX,NX),VP(LX,MX,NX),TP(LX,MX,NX),PP(LX,MX),
     +     DU(LX,MX,NX),DV(LX,MX,NX),DT(LX,MX,NX),DP(LX,MX)
      CALL INTEE1 (LX,MX,NX,DTM,DU,UP,U)
      CALL INTEE1 (LX,MX,NX,DTM,DV,VP,V)
      CALL INTEE1 (LX,MX,NX,DTM,DT,TP,T)
      CALL INTEE1 (LX,MX, 1,DTM,DP,PP,P)
      RETURN
      END

      SUBROUTINE INTEL1 (LX,MX,NX,DTM,DG,GP,G)
      REAL DG(LX,MX,NX),GP(LX,MX,NX),G(LX,MX,NX)
      EF = 0.5E-1
      DO 11 N=1,NX
      DO 12 M=1,MX
      DO 13 L=1,LX
        GP1       = GP(L,M,N)
        GP(L,M,N) = ((1-0.2E1*EF)*G(L,M,N) + EF*GP(L,M,N)) / (1.0E0-EF)
        G (L,M,N) = GP1 + 0.2E1*DTM * DG(L,M,N)
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE INTEE1 (LX,MX,NX,DTM,DG,GP,G)
      REAL DG(LX,MX,NX),GP(LX,MX,NX),G(LX,MX,NX)
      EF = 0.5E-1
      DO 11 N=1,NX
      DO 12 M=1,MX
      DO 13 L=1,LX
        G (L,M,N) = G(L,M,N) + 0.2E1*DTM * DG(L,M,N)
        GP(L,M,N) = (1.0E0-EF)*GP(L,M,N) + EF*G(L,M,N)
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END
